/*
 *  io.h
 *
 *  Created by samel on 16/05/07.
 *  Copyright 2007 DoNotJava. All rights reserved.
 *
 */

#include "color.h"

//brief: debug trick
#ifdef DEBUG_IO_ENABLE
#define DEBUG_IO(S,A) {_set_color(S,BROWN,BLACK);_print A;_set_color(S,GREY,BLACK);}
#else
#define DEBUG_IO(S,A)
#endif

//brief: video buffer for pixel coloring
#define VIDEO_BUFFER ((unsigned short int*)BG_BMP_RAM(0))
#define SUB_VIDEO_BUFFER ((unsigned short int*)BG_BMP_RAM_SUB(0))

//brief: video name for user
#define UPPER 1
#define LOWER 2

//brief:console base color
#define CONSOLE_FGCOLOR GREY
#define CONSOLE_BGCOLOR BLACK

//brief:console heigth and width
#define CONSOLE_LINE 23
#define CONSOLE_ROW 31

//brief:screen size in char
#define SCREEN_SIZE_CHAR CONSOLE_LINE*CONSOLE_ROW

//brief:font definition
#define COLOR_CELL 1

//brief:tabulation width definition
#define TABULATION_WIDTH 2

/*
 brief: initializes the console in the upper screen
 return: always 1
 */
int _top_console_init(void);

/*
 brief: initializes the console in the lower screen
 return: always 1
 */
int _bottom_console_init(void);

/*
 brief: color a pixel on the screen
 param: screen, UPPER or LOWER
 param: x, the x coord
 param: y, the y coord
 param: color, the pixel color in 15 bit notation
 */
void _print_pixel(char screen,unsigned char x,unsigned char y,unsigned short int color);

/*
 brief: print a char on the screen
 param: screen, UPPER or LOWER
 param: toprint, character to print
 return: the character printed 
 */
unsigned char _put_char(char screen,unsigned char toprint);

/*
 brief: print a char on the screen with \n and \t management
 param: screen, UPPER or LOWER
 param: toprint, character to print
 return: the character printed 
 */
unsigned char _print_char(char screen,unsigned char toprint);

/*
 brief: print an integer with sign on the screen
 param: screen, UPPER or LOWER
 param: integer, number to print
 return: the integer printed 
 */
signed int _print_int(char screen,signed int integer);

/*
 brief: print the hexadecimal conversion of an integer with sign on the screen
 param: screen, UPPER or LOWER
 param: hexadecimal, number to convert and print
 return: the integer printed 
 */
signed int _print_hex(char screen,signed int hexadecimal);

/*
 brief: print a character sequence on the screen
 param: screen, UPPER or LOWER
 param: string, character sequence to print
 return: number of printed character
 */
unsigned int _print_string(char screen,unsigned char* string);

/*
 brief: print a formatted string on the screen
 param: screen, UPPER or LOWER
 param: string, format string
 param: ..., optional parameters depending on the format
 format: %s string, %c character, %d integer, %h hexadecimal
 return: number of printed token 
 */
unsigned int _print(char screen,char* string,...);

/*
 brief: move the cursor to the specified position
 param: screen, UPPER or LOWER
 param: x, the x coord in character coloumn
 param: y, the y coord in character row
 return: 1 if successfully, 0 otherwise
 */
unsigned char _goto_xy(char screen,unsigned char x,unsigned char y);

/*
 brief: set the height and the width of the screen
 param: screen, UPPER or LOWER
 param: width, the screen width
 param: height, the screen height
 return: 1 if successfully, 0 otherwise
 */
unsigned char _set_screen(char screen,unsigned char width,unsigned char height);

/*
 brief: set the foreground and the background color of the screen
 param: screen, UPPER or LOWER
 param: fg, the foreground color in 15 bit notation
 param: bg, the background color in 15 bit notation
 */
void _set_color(char screen,unsigned short int fg,unsigned short int bg);

/*
 brief: clear the entire screen
 param: screen, UPPER or LOWER
 return: the number of console line
 */
unsigned int _clear_screen(char screen);

/*
 brief: check if a new line is needed
 param: screen, UPPER or LOWER
 return: 1 if new line added, 0 otherwise
 */
unsigned char _check_return(char screen);

/*
 brief: check if a scroll up is needed
 param: screen, UPPER or LOWER
 return: 1 if screen scrolled up, 0 otherwise
 */
unsigned char _check_end(char screen);

/*
 brief: move the cursor forward a single character
 param: screen, UPPER or LOWER
 return: always 1 
 */
unsigned char _line_forward(char screen);

/*
 brief: move the cursor backward a single character
 param: screen, UPPER or LOWER
 return: 1 if successfully, 0 otherwise
 */
unsigned char _line_backward(char screen);

/*
 brief: restore the cursor position from the backup
 param: screen, UPPER or LOWER
 */
void _restore_cursor(char screen);

/*
 brief: backup the cursor position
 param: screen, UPPER or LOWER
 */
void _backup_cursor(char screen);

/*
 brief: set current screen
 param: screen, UPPER or LOWER
 */
void _set_current_screen(char screen);

/*
 brief: save state of current screen
 param: screen, UPPER or LOWER
 */
void _update_screen(char screen);
