/*
 *  timer.h
 *
 *  Created by samel on 12/05/07.
 *  Copyright 2007 DoNotJava. All rights reserved.
 *
 */

//brief: debug trick
#ifdef DEBUG_TIMER_ENABLE
#define DEBUG_TIMER(S,A) {_set_color(S,YELLOW,BLACK);_print A;_set_color(S,GREY,BLACK);}
#else
#define DEBUG_TIMER(S,A)
#endif

/*
 TIMER0 is used to handle the arm7-arm9 FIFO data transfer for network
 comunication
 TIMER1 is used for broadcast comunication
 TIMER2 is used for tcp server comunication
 */
#define TIMER0 0
#define TIMER1 1
#define TIMER2 2
#define TIMER3 3
#define DIV1    0
#define DIV64   1
#define DIV256  2
#define DIV1024 3

/*
 brief: initializes a timer at a certain frequency with a defined function
 param: number, the timer identifier
 param: freq, the timer frequency
 [Max: 33554432Hz
  Min: 512Hz]
 param: div, the frequency divider
 [DIV1 => timer counts at 33.514Mhz.
  DIV64 => timer counts at (33.514 / 64) Mhz.
  DIV256 => timer counts at (33.514 / 256) Mhz.
  DIV1024 => timer counts at (33.514 / 1024) Mhz.]
 param: function, the timer associated function
  [the function will be called each time the timer overflows]
 return: 1 on timer enabled, -1 otherwise
 */
int _timer_start(int number,int freq,int div,void* function);

/*
 brief: deinitializes a timer
 param: number, the timer identifier
 return: 1 on timer enabled, -1 otherwise
 */
int _timer_stop(int number);

/*
 brief: function called when time out occurs
 */
void _set_time_out(void);
