/*
 *  network.c
 *
 *  Created by samel on 12/05/07.
 *  Copyright 2007 DoNotJava. All rights reserved.
 *
 */

#include "header.h"
#include <nds.h>
#include <dswifi9.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <string.h>
#include <stdio.h>
#include "arm7.h"
#include "interrupt.h"
#include "io.h"
#include "network.h"
#include "timer.h"

//brief: localhost ip
char _local_ip[15]="192.168.002.100";
//char _local_ip[15]="192.168.002.101";
//char _local_ip[15]="192.168.002.102";
//brief: local netmask
char _local_mask[15]="255.255.255.000";
//char _local_mask[15]="000.000.000.000";
//brief: local ssid
//char _local_ssid[15]="Callidus";
char _local_ssid[15]="ANY";
//brief: local mac address
unsigned char _local_mac[6];
//brief: local wep key
unsigned char _wep_key[4][32];
//brief: local ap
Wifi_AccessPoint ap;

//from: timer.c
extern int _time_out;

int _network_init(void)
{
 DEBUG_NETWORK(UPPER,(UPPER,"Network Initialization\nnetwork ip:%s\nnetwork mask:%s\nnetwork ssid:%s\n",_local_ip,_local_mask,_local_ssid));
 bzero(_local_mac,sizeof(_local_mac));
 bzero(_wep_key,sizeof(_wep_key));
 bzero(&ap,sizeof(ap));
 REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_SEND_CLEAR;
 u32 Wifi_pass = Wifi_Init(WIFIINIT_OPTION_USELED);
 REG_IPC_FIFO_TX=0x12345678;
 REG_IPC_FIFO_TX=Wifi_pass;
 _timer_start(TIMER0,20,DIV256,_wifi_timer);
 irqSet(IRQ_FIFO_NOT_EMPTY, arm9_fifo);
 irqEnable(IRQ_FIFO_NOT_EMPTY);
 REG_IPC_FIFO_CR = IPC_FIFO_ENABLE | IPC_FIFO_RECV_IRQ;
 Wifi_SetSyncHandler(arm9_synctoarm7);
 while(Wifi_CheckInit()==0) swiWaitForVBlank();
 irqSet(IRQ_VBLANK,Wifi_Update);
 Wifi_EnableWifi();
 strcpy(ap.ssid,_local_ssid);
 if(_connect()<0)
  return -1;
 Wifi_SetIP(inet_addr(_local_ip),0,inet_addr(_local_mask),0,0);
 return _socket_init();
}
//------------------------------------------------------------------------------
int _connect(void)
{
 Wifi_ConnectAP(&ap, WEPMODE_NONE, 0, _wep_key[0]);
 _time_out=1;
 _timer_start(TIMER3,0.5,DIV1024,(void*)_set_time_out);
 //while not associated to the AP
 while (Wifi_AssocStatus()!=ASSOCSTATUS_ASSOCIATED)
 {
  //if cannot connect to AP
  if(Wifi_AssocStatus()==ASSOCSTATUS_CANNOTCONNECT || _time_out==0)
  {
   _timer_stop(TIMER3);
   _network_deinit();
   DEBUG_NETWORK(UPPER,(UPPER,"network error\n"));
   return -1;
  }
 }
 _timer_stop(TIMER3);
 return 1;
}
//------------------------------------------------------------------------------
int _network_deinit(void)
{
 DEBUG_NETWORK(UPPER,(UPPER,"Network deinitialization\n"));
 Wifi_DisconnectAP();
 _timer_stop(TIMER0);
 Wifi_DisableWifi();
 return 1;
}
//------------------------------------------------------------------------------
int _get_network_signal()
{
 DEBUG_NETWORK(UPPER,(UPPER,"Network get signal\n"));
 int i=0;
 Wifi_AccessPoint ap_tmp;
 Wifi_ScanMode();
 bzero(&ap_tmp,sizeof(ap_tmp));
 strcpy(ap_tmp.ssid,_local_ssid);
 i=Wifi_FindMatchingAP(1,&ap_tmp,NULL);
 Wifi_GetAPData(i,&ap_tmp);
 if(_connect()<0)
 {
  _network_deinit();
  return -1;
 }
 DEBUG_NETWORK(UPPER,(UPPER,"signal strength %d %%\n",(ap_tmp.rssi*100)/0xD0)); 
 return (ap_tmp.rssi*100)/0xD0;
}
//------------------------------------------------------------------------------
int _socket_init(void)
{
 DEBUG_NETWORK(UPPER,(UPPER,"Socket initialization\n"));
 return 1;
}
//------------------------------------------------------------------------------
void _wifi_timer(void)
{
 //DEBUG_NETWORK(UPPER,(UPPER,"-Wifi timer loop\n")); // too much output
 Wifi_Timer(50);
}
//------------------------------------------------------------------------------
char* _get_local_ssid()
{
 //DEBUG_NETWORK(UPPER,(UPPER,"-Getting local ssid\n")); // not useful
 return _local_ssid;
}
//------------------------------------------------------------------------------
unsigned char* _get_local_mac()
{
 //DEBUG_NETWORK(UPPER,(UPPER,"-Getting local mac\n")); // not useful
 return _local_mac;
}
//------------------------------------------------------------------------------
char* _get_local_ip()
{
 //DEBUG_NETWORK(UPPER,(UPPER,"-Getting local ip\n")); // not useful
 return _local_ip;
}
//------------------------------------------------------------------------------
char* _get_local_mask()
{
 //DEBUG_NETWORK(UPPER,(UPPER,"-Getting local mask\n")); // not useful
 return _local_mask;
}
